#!/bin/sh

# Set kernel parameters that fix clock drift.
# Use for RHEL guests running on KVM.

ARCH=$(uname -m)
GRUB_CONF=/boot/grub/grub.conf
GRUB_CONF_KVM=${GRUB_CONF}.kvm
VENDOR=$(lshal | grep -i "system.vendor" | grep "Red Hat")
CLOCKSOURCE=$(cat /sys/devices/system/clocksource/clocksource0/current_clocksource)

cat <<EOF
This script will add kernel command line parameters to fix clock drift for kvm 
guests. It will at first make some tests and then it will add the parameters.

The script will not modify "${GRUB_CONF}" directly. It will copy it
to "${GRUB_CONF_KVM}" and modify this file.

When the script completes, please check the this copy to ensure that the 
contents are correct and rename it to "${GRUB_CONF}" to activate the 
changes. It might be good to make a backup copy of the original before 
overwriting it.

EOF

echo -n "Do you want to continue? [y/N]: "
read -e ANSWER
if [ "$ANSWER" != "y" ]; then
    echo "Exiting."
    exit 0;
fi
echo

case ${ARCH} in
    x86_64)
	PARAMS="divider=10 notsc"
	;;
    i686)
	PARAMS="divider=10 clocksource=acpi_pm"
	;;
    *)
	echo "The architecture \"${ARCH}\" is not supported by this script."
	exit 1
	;;
esac

if [ ! -r "${GRUB_CONF}" ]; then
    echo "ERROR: ${GRUB_CONF}: Permission denied."
    exit 1
fi

if [ -z "${VENDOR}" ]; then
    echo "ERROR: This is no kvm guest, exiting."
    exit 1
fi

# x86_64 specific test
if dmesg | grep -q "time\.c:.*MHz WALL KVM GTOD KVM timer"; then
    echo "ERROR: KVM timer already in use, exiting."
    exit 1
fi

# i686 specific test (for now?)
if [ "${CLOCKSOURCE%% }x" = "kvm-clockx" ]; then
    # CLOCKSOURCE has/could have a leading space
    echo "ERROR: KVM timer already in use, exiting."
    exit 1
fi

if grep -q -E "^[[:space:]]+kernel.*(lpj=)|(divider=)|(notsc)" "${GRUB_CONF}"; then
    echo "ERROR: \"${GRUB_CONF}\" already contains clock drift fixes, exiting."
    exit 1
fi

echo "Creating copy \"${GRUB_CONF_KVM}\" .."

cp "${GRUB_CONF}" "${GRUB_CONF_KVM}" || exit 1

echo "Adding \"${PARAMS}\" to the kernel command line(s) .."

sed -i -e "s:^[[:space:]]\+kernel.*:\0 ${PARAMS}:g" "${GRUB_CONF_KVM}"

cat <<EOF

Please check the file "${GRUB_CONF_KVM}" to ensure that the contents 
are correct and rename it to "${GRUB_CONF}".

Added command line parameters: "${PARAMS}"
EOF
